﻿using ASHermed.SinoInterface.Api.ContractModel;
using ASHermed.SinoInterface.Api.ContractModel.App;
using ASHermed.SinoInterface.Api.IRepository;
using ASHermed.SinoInterface.Api.IService;
using ASHermed.SinoInterface.IFramework;
using ASHermed.SinoInterface.Tools;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using ASHermed.SinoInterface.Api.ContractModel.App.Entity;
using ASHermed.SinoInterface.Api.ContractModel.TCTrans.Input;
using ASHermed.SinoInterface.Api.IRepository.App;
using ASHermed.SinoInterface.Api.ContractModel.App.RequestForm;
using ASHermed.SinoInterface.Api.MySqlRepository.App;
using System.IO;
using NPOI.POIFS.Properties;
using TencentCloud.Cme.V20191029.Models;

namespace ASHermed.SinoInterface.Api.Service
{
    /// <summary>
	/// 老用户信息
	/// </summary>
	public class PatService : BaseService, IPatService
    {
        #region 初始化
        IPubPatInfoRepository _pubPatRepository;
        IPubMapUserPatRepository _pubMapUserPatRepository;
        ITCService _tCService;
        IAptService _aptService;
        ILogger<VerifyCodeLogService> logger;
        IPubUserInfoRepository _pubUserInfoRepository;
        IOutService _outService;
        private readonly IZhuTongSMSService _zhuTongSmsService;
        private readonly IPubPatWashDataRepository _pubPatWashDataRepository;
        private readonly IPubAuthorizationRepository _pubAuthorizationRepository;
        IOnlineOrderRepository _onlineOrderRepository;
        public PatService(IPubPatInfoRepository pubPatRepository, IPubMapUserPatRepository pubMapUserPatRepository, ITCService arPatService, ILogger<VerifyCodeLogService> _logger, IAptService aptService, IPubUserInfoRepository pubUserInfoRepository, IOutService outService, IZhuTongSMSService zhuTongSmsService, IPubPatWashDataRepository pubPatWashDataRepository, IPubAuthorizationRepository pubAuthorizationRepository, IOnlineOrderRepository onlineOrderRepository)
        {
            _pubPatRepository = pubPatRepository;
            _pubMapUserPatRepository = pubMapUserPatRepository;
            _tCService = arPatService;
            logger = _logger;
            _aptService = aptService;
            _pubUserInfoRepository = pubUserInfoRepository;
            _outService = outService;
            _zhuTongSmsService = zhuTongSmsService;
            _pubPatWashDataRepository = pubPatWashDataRepository;
            _pubAuthorizationRepository = pubAuthorizationRepository;
            _onlineOrderRepository = onlineOrderRepository;
        }
        #endregion

        public List<Pub_Pat_Info> GetHisPat(string mrn, string phone, string birthday)
        {
            List<PAPerson> pat = _tCService.GetPatient(mrn, phone, birthday);
            List<Pub_Pat_Info> list = new List<Pub_Pat_Info>();
            foreach (var item in pat)
            {
                PatWithAge patInfo = new PatWithAge();
                patInfo.Pat_Source = 1;
                patInfo.Is_Deleted = false;
                patInfo.Birthday = item.PAPERDob + "";
                patInfo.Age = item.PAPERDob.GetAgeByBirth();
                patInfo.Email = item.PAPEREmail;
                patInfo.Last_Name = item.PAPERName;
                patInfo.First_Name = item.PAPERName2;
                patInfo.Gender = item.PAPERSex.ToLower().Contains("female") ? 0 : 1;
                patInfo.Is_Active = item.PAPMIActive == "Y" ? true : false;
                patInfo.Is_Dead = item.PAPERDeceased == "Y" ? true : false;
                patInfo.Pat_ID = item.PAPERRowId;
                patInfo.Pat_MRN = item.PAPMINo;
                patInfo.Nationality = item.PAPERNation;
                patInfo.Card_ID = item.PAPERID;
                if (patInfo.Card_ID != null)
                {
                    if (patInfo.Card_ID.Length == 15 || patInfo.Card_ID.Length == 18)
                    {
                        patInfo.Card_Type = 1;
                    }
                    else
                    {
                        patInfo.Card_Type = 2;
                    }
                }

                if (!string.IsNullOrEmpty(patInfo.Last_Name) && patInfo.Last_Name.IsChinese())
                {
                    patInfo.Pat_Name = patInfo.Last_Name + patInfo.First_Name;
                }
                else
                {
                    patInfo.Pat_Name = patInfo.First_Name + patInfo.Last_Name;
                }
                if (!string.IsNullOrEmpty(item.PAPERMobPhone))
                {
                    patInfo.Phone_NO = item.PAPERMobPhone;
                }
                else
                {
                    patInfo.Phone_NO = phone;
                }
                SaveHisPat(patInfo);
                list.Add(patInfo);
            }

            return list;
        }

        public int CancelUserDefult(int userId)
        {
            return _pubPatRepository.CancelPatDefult(userId);
        }


        public int SaveHisPat(Pub_Pat_Info patInfo)
        {
            if (patInfo.Card_Type == 1)
            {
                patInfo.Nationality = "CN";
            }
            Pub_Pat_Info pubPatInfo = _pubPatRepository.SelectByPatId(patInfo.Pat_ID);
            if (null != pubPatInfo)
            {
                pubPatInfo.Birthday = Convert.ToDateTime(patInfo.Birthday).ToString("yyyy-MM-dd");
                pubPatInfo.Is_Deleted = false;
                pubPatInfo.Pat_Source = patInfo.Pat_Source;
                pubPatInfo.Email = patInfo.Email;
                pubPatInfo.Last_Name = patInfo.Last_Name;
                pubPatInfo.First_Name = patInfo.First_Name;
                pubPatInfo.Gender = patInfo.Gender;
                pubPatInfo.Is_Active = patInfo.Is_Active;
                pubPatInfo.Is_Dead = patInfo.Is_Dead;
                pubPatInfo.Pat_ID = patInfo.Pat_ID;
                pubPatInfo.Pat_MRN = patInfo.Pat_MRN;
                pubPatInfo.Nationality = patInfo.Nationality;
                pubPatInfo.Card_ID = patInfo.Card_ID;
                pubPatInfo.Card_Type = patInfo.Card_Type;
                pubPatInfo.Pat_Name = patInfo.Pat_Name;
                pubPatInfo.Phone_NO = patInfo.Phone_NO;
                _pubPatRepository.UpdatePubPatInfo(pubPatInfo);
                patInfo.Id = pubPatInfo.Id;
                //_pubPatRepository.UpdatePubPatInfo(patInfo);
            }
            else
            {
                patInfo.Birthday = Convert.ToDateTime(patInfo.Birthday).ToString("yyyy-MM-dd");
                patInfo.Is_Deleted = false;
                patInfo.Pat_Source = 1;
                patInfo.Id = _pubPatRepository.InsertPubPatInfo(patInfo);
                //_pubPatRepository.InsertPubPatInfo(patInfo);
            }
            return patInfo.Id;
        }

        public string GetPatNameByLanguage(string lastName, string firstName, int language)
        {
            string patName = null;
            if (!firstName.IsChinese())
            {
                patName = firstName + " " + lastName;
            }
            else
            {
                patName = lastName + firstName;
            }
            return patName;
        }

        public Pub_Pat_Info CheckPatIsExist(Pub_Pat_Info rpos)
        {
            var pat = _pubPatRepository.GetPatByCondition(rpos.Phone_NO, rpos.Last_Name, rpos.First_Name);
            if (pat != null)
            {
                rpos.Id = pat.Id;
            }
            return rpos;
        }

        public Pub_Pat_Info Add(Pub_Pat_Info patInfo, int relationShip, int language)
        {
            var userId = GetBaseUserInfo().ID;
            if (patInfo.IsDefault == 1)
            {
                CancelUserDefult(userId);
            }
            if (patInfo.Card_Type == 1)
            {
                patInfo.Nationality = "CN";
            }
            // 新增或修改患者信息
            int patId = patInfo.Id;
            string patName = GetPatNameByLanguage(patInfo.Last_Name, patInfo.First_Name, language);
            if (patName != null)
            {
                patInfo.Pat_Name = patName;
            }
            Pub_Pat_Info pubPatInfo = _pubPatRepository.SelectById(patInfo.Id);
            if (null == pubPatInfo)
            {
                patInfo.Birthday = Convert.ToDateTime(patInfo.Birthday).ToString("yyyy-MM-dd");
                patInfo.Is_Deleted = false;
                patInfo.Pat_Source = 0;

                var pprid = _pubPatRepository.InsertPubPatInfo(patInfo);
                patInfo.Id = int.Parse(pprid.ToString());
                //_pubPatRepository.InsertPubPatInfo(patInfo);
            }
            else
            {
                patInfo.Birthday = Convert.ToDateTime(patInfo.Birthday).ToString("yyyy-MM-dd");
                patInfo.Id = pubPatInfo.Id;
                var Repository = _pubPatRepository.UpdatePubPatInfo(patInfo);
                //_pubPatRepository.UpdatePubPatInfo(patInfo);
            }

            Pub_Map_User_Pat pubMapUserPat = null;
            if (patId != patInfo.Id)
            {
                // 这里先让原来的关系隐藏
                pubMapUserPat = _pubMapUserPatRepository.GetPubUserPat(userId, patId);
                if (pubMapUserPat != null)
                {
                    pubMapUserPat.Relation_Ship = pubMapUserPat.Relation_Ship | 8;
                    _pubMapUserPatRepository.UpdatePubMapUserPat(pubMapUserPat);
                }
                //_pubMapUserPatRepository.UpdatePubMapUserPat(pubMapUserPat);
            }

            pubMapUserPat = _pubMapUserPatRepository.GetPubUserPat(userId, patInfo.Id);

            if (null == pubMapUserPat)
            {
                pubMapUserPat = new Pub_Map_User_Pat()
                {
                    User_ID = userId,
                    Pat_ID = patInfo.Id,
                    Is_Deleted = false,
                    Relation_Ship = relationShip,
                    Create_Time = DateTime.Now

                };
                _pubMapUserPatRepository.InsertPubMapUserPat(pubMapUserPat);
                //_pubMapUserPatRepository.InsertPubMapUserPat(pubMapUserPat);
            }
            else
            {
                pubMapUserPat.Pat_ID = patInfo.Id;
                pubMapUserPat.User_ID = userId;
                pubMapUserPat.Update_Time = DateTime.Now;
                pubMapUserPat.Relation_Ship = pubMapUserPat.Relation_Ship | relationShip;
                _pubMapUserPatRepository.UpdatePubMapUserPat(pubMapUserPat);
                //_pubMapUserPatRepository.UpdatePubMapUserPat(pubMapUserPat);
            }

            return patInfo;
        }



        public bool UpdatePubPatInfo(PatWithRelationShip patInfo)
        {
            if (patInfo.IsDefault == 1)
            {
                CancelUserDefult(GetBaseUserInfo().ID);
            }
            return _pubPatRepository.UpdatePubPatInfo(patInfo) > 0 ? true : false;
        }

        public List<PatWithRelationShip> FindBindingsTime(int _userid, string type = "")
        {
            if (GetBaseUserInfo() == null)
            {
                return new List<PatWithRelationShip>();
            }

            var userid = GetBaseUserInfo().ID;

            //logger.LogError("userid----------" + userid);
            List<PatWithRelationShip> list = _pubPatRepository.SelectByUserWithRelationTime(userid, type);
            list = list.FindAll(p => p.RelationShip == 1 || p.RelationShip == 2);
            if (list != null && list.Count > 0)
            {
                var authorizationInfos = _pubAuthorizationRepository.GetUserAuthorizationsByPats(userid, list.Select(x => x.Id).ToList());

                foreach (var item in list)
                {
                    var aptInfo = _aptService.FindLastAptByPat(item.PatId);
                    if (aptInfo != null)
                    {
                        item.RecentAptTime = aptInfo.FirstOrDefault()?.Apt_Time;
                    }
                    DateTime dt = DateTime.MinValue.Date;
                    DateTime.TryParse(item.Birthday, out dt);
                    item.Birthday = dt.ToString("yyyy-MM-dd");
                    //Convert.ToDateTime(item.Birthday).ToString("yyyy-MM-dd");

                    item.Age = Convert.ToDateTime(item.Birthday).GetAgeByBirth();

                    //头像显示类型：1-男；2-女；3-小于14岁
                    if (item.Age > 14)
                    {
                        item.HeadImgType = item.Gender == 1 ? 1 : 2;
                    }
                    else
                    {
                        item.HeadImgType = 3;
                    }

                    if (item.Ties == 1 || item.Age < 18)
                    {
                        item.AuthorizationStatus = 3;
                        item.AuthorizationDetailIds = "1,2,3,4";
                    }
                    else
                    {
                        var auth = authorizationInfos.Where(x => x.PatId == item.Id)
                                                     .OrderByDescending(x => x.AuthorizationValidity)
                                                     .FirstOrDefault();

                        if (auth == null)
                        {
                            continue;
                        }

                        var now = DateTime.Now;
                        item.AuthorizationStatus = auth.Status == 3 && (!auth.IsForever && auth.AuthorizationValidity < now) ? 5 : auth.Status;
                        item.AuthorizationDetailIds = auth.Status == 3 && (auth.IsForever || auth.AuthorizationValidity > now) ? auth.ApplyDetailIds : default;
                    }

                }

                //"本人"排第一
                var mySelf = list.FirstOrDefault(x => x.Ties == 1);
                if (mySelf != null)
                {
                    list.Remove(mySelf);
                    list.Insert(0, mySelf);
                }
            }
            return list;
        }


        public List<PatWithRelationShip> FindRelationPatListByApp()
        {
            List<PatWithRelationShip> list = _pubPatRepository.SelectByUserWithRelationByApp(GetBaseUserInfo().ID);
            list = list.FindAll(p => p.RelationShip == 1 || p.RelationShip == 2);

            //“本人”排第一个
            var mySelf = list.FirstOrDefault(x => x.Ties == 1);
            if (mySelf != null)
            {
                list.Remove(mySelf);
                list.Insert(0, mySelf);
            }
            //list = list.stream().filter(o->(RelationShipType.UNDISPLAY.getValue() & (int)o.getRelationShip()) == 0).collect(Collectors.toList());
            var authorizationInfos = _pubAuthorizationRepository.GetUserAuthorizationsByPats(GetBaseUserInfo().ID, list.Select(x => x.Id).ToList());
            foreach (var item in list)
            {
                try
                {
                    item.Age = Convert.ToDateTime(item.Birthday).GetAgeByBirth();
                }
                catch (System.Exception e) { }

                //头像显示类型：1-男；2-女；3-小于14岁
                if (item.Age > 14)
                {
                    item.HeadImgType = item.Gender == 1 ? 1 : 2;
                }
                else
                {
                    item.HeadImgType = 3;
                }

                if (item.Ties == 1 || item.Age < 18)
                {
                    item.AuthorizationStatus = 3;
                    item.AuthorizationDetailIds = "1,2,3,4";
                }
                else
                {
                    var auth = authorizationInfos.Where(x => x.PatId == item.Id)
                                                 .OrderByDescending(x => x.AuthorizationValidity)
                                                 .FirstOrDefault();

                    if (auth == null)
                    {
                        continue;
                    }

                    var now = DateTime.Now;
                    item.AuthorizationStatus = auth.Status == 3 && (!auth.IsForever && auth.AuthorizationValidity < now) ? 5 : auth.Status;
                    item.AuthorizationDetailIds = auth.Status == 3 && (auth.IsForever || auth.AuthorizationValidity > now) ? auth.ApplyDetailIds : default;
                }
            }
            return list;
        }

        public List<PatWithRelationShip> FindRelationPatListByAppAppt()
        {
            List<PatWithRelationShip> list = _pubPatRepository.SelectByUserWithRelationByApp(GetBaseUserInfo().ID);

            //“本人”排第一个
            var mySelf = list.FirstOrDefault(x => x.Ties == 1);
            if (mySelf != null)
            {
                list.Remove(mySelf);
                list.Insert(0, mySelf);
            }

            var authorizationInfos = _pubAuthorizationRepository.GetUserAuthorizationsByPats(GetBaseUserInfo().ID, list.Select(x => x.Id).ToList());

            foreach (var item in list)
            {
                try
                {
                    item.Age = Convert.ToDateTime(item.Birthday).GetAgeByBirth();
                }
                catch (System.Exception e) { }
                //头像显示类型：1-男；2-女；3-小于14岁
                if (item.Age > 14)
                {
                    item.HeadImgType = item.Gender == 1 ? 1 : 2;
                }
                else
                {
                    item.HeadImgType = 3;
                }

                if (item.Ties == 1 || item.Age < 18)
                {
                    item.AuthorizationStatus = 3;
                    item.AuthorizationDetailIds = "1,2,3,4";
                }
                else
                {
                    var auth = authorizationInfos.Where(x => x.PatId == item.Id)
                                                 .OrderByDescending(x => x.AuthorizationValidity)
                                                 .FirstOrDefault();

                    if (auth == null)
                    {
                        continue;
                    }

                    var now = DateTime.Now;

                    item.AuthorizationStatus = auth.Status == 3 && !auth.IsForever && auth.AuthorizationValidity < now ? 5 : auth.Status;
                    item.AuthorizationDetailIds = auth.Status == 3 && (auth.IsForever || auth.AuthorizationValidity > now) ? auth.ApplyDetailIds : default;
                }
            }

            return list;
        }

        public Pub_Pat_Info GetPatInfo(int patId)
        {
            Pub_Pat_Info pubPatInfo = _pubPatRepository.SelectById(patId);
            return pubPatInfo;
        }

        public List<Pub_Pat_Info> GetUserRelationship()
        {
            return _pubPatRepository.GetUserRelationship(GetBaseUserInfo().ID);
        }
        public string Bind2User(int userId, string patCode, int relationShip, int language)
        {
            userId = GetBaseUserInfo().ID;
            Pub_Pat_Info pubPatInfo = _pubPatRepository.SelectByPatId(patCode);
            if (null == pubPatInfo)
            {
                return language == 2 ? "the patient is not found" : "被绑定就诊人不存在";
            }

            // 判断该患者是否被其他用户绑定
            if (pubPatInfo != null)
            {
                List<Pub_Map_User_Pat> listMapUserPat = _pubMapUserPatRepository.SelectByPatId(pubPatInfo.Id);
                if (listMapUserPat.Find(p => p.Relation_Ship == 1) != null)
                {
                    return language == 2 ? "the patient has been bound to other user" : "被绑定就诊人已经被其他用户绑定";
                }
            }

            Add(pubPatInfo, 1, language);

            Pub_User_Info userInfo = _pubUserInfoRepository.SelectByPrimaryKey(userId);
            if (userInfo != null && !string.IsNullOrEmpty(userInfo.Open_ID))
            {
                if (relationShip == 1)
                {
                    //发送互联网医院绑定成功通知
                    string json = "{\n" +
                            "    \"openid\": \"" + userInfo.Open_ID + "\",\n" +
                            "    \"patientName\": \"" + pubPatInfo.Pat_Name + "\",\n" +
                            "    \"tel\": \"" + pubPatInfo.Phone_NO + "\"\n" +
                            "}";
                    _outService.HttpPost("http://106.14.207.193:8191/RegistMedicalCard.ashx?action=add", json);
                    //HttpUtil.post(INTERNET_ADD_PAT_URL, json);
                }
                if (relationShip == 8)
                {
                    //发送互联网医院绑定成功通知
                    string json = "{\n" +
                            "    \"openid\": \"" + userInfo.Open_ID + "\",\n" +
                            "    \"patientName\": \"" + pubPatInfo.Pat_Name + "\",\n" +
                            "    \"tel\": \"" + pubPatInfo.Phone_NO + "\"\n" +
                            "}";
                    _outService.HttpPost("http://106.14.207.193:8191/RegistMedicalCard.ashx?action=delete", json);
                    //HttpUtil.post(INTERNET_DEL_PAT_URL, json);
                }


            }
            //发送通知到TC，患者已经注册PP
            if (!string.IsNullOrEmpty(pubPatInfo.Pat_MRN))
            {
                _tCService.UpdatePatientPortalLink("", pubPatInfo.Pat_MRN, true);
            }
            return "";
        }




        public void MarkingPat()
        {
            const string content = @"尊敬的健康俱乐部会员，您订阅的俱乐部内容更新啦！请前往查阅：zt81.cn/SWmM6
Dear Wellness Club Member, your subscription is updated! Please click the link and check the latest : zt81.cn/SWmM6";
            var tempphones = new List<string>
            {
                "17814663133",
            };

            var igoreList = new List<string>
            {
                "18616555857",
                "	 17621619619	",
                "	 18516311858	",
                "	 13140400366	",
                "	 17602045272	",
                "	 15797869945	",
                "	 18816290701	",
                "	15957288391	",
                "	15902117380	",
                "	15156365221	",
                "	13734028655	",
                "	13122159376	",
                "	15797878496	",
                "	13231490258	",
                "	13122140305	",
                "	13382126326	",
                "	15705185776	",
                "	17656685350	",
                "	17843096602	",
                "	18851648652	",
                "	17801063346	",
                "	18921341235	",
                "	18182260353	",
                "	16621003491	",
                "	18761296237	",
                "	18391155299	",
                "	17351012019	",
                "	18817358236	",
                "	17864288496	",
                "	15252159072	",
                "	19821216620	",
                "	18816209452	",
                "	15026506014	",
                "	18505540163	",
                "	15695262628	",
                "	18382189236	",
                "	13482267869	",
                "	15370028732	",
                "	13946539316	",
                "	15505165933	",
                "	18261193319	",
                "	15908911956	",
                "	13918204020	",
                "	174289958"    ,
                "	17878760681	",
                "	18133679142	",
                "	15506293073	",
                "	18670339731	",
                "	15704616519	",
                "	18101732676	",
                "	15802789366	",
                "	18021069005	",
                "	18827555151	",
                "	17879612437	",
                "	13636548289	",
                "	13956084130	",
                "	15599039585	",
                "	18270975880	",
                "	13916533794	",
                "	13914503032	",
                "	18550395919	",
                "	15702410625	",
                "	15852918087	",
                "	13840802107	",
                "	18357143636	",
                "	19512393509	",
                "	15557305183	",
                "	13817345129	",
                "	15206284109	",
                "	13701918780	",
                "	17521197492	",
                "	13160380288	",
                "	19946279005	",
                "	13370018975	",
                "	18895337816	",
                "	18903390125	",
                "	15555629015	",
                "	17640172185	",
                "	18397914969	",
                "	18360428796	",
                "	15821745354	",
                "	13183017141	",
                "	15021227279	",
                "	18879733523	",
                "	18017323801	",
                "	15951605356	",
                "	18278385060	",
                "	13671630518	",
                "	15394239289	",
                "	18362798420	",
                "	15834477299	",
                "	13576110579	",
                "	17531379529	",
                "	15618743686	",
                "	15929920565	",
                "	18762993209	",
                "	13761293958	",
                "	18262632939	",
                "	13818366574	",
                "	18651600979	",
                "	17610785165	",
                "	18621719822	",
                "	15264387675	",
                "	19967649948	",
                "	17521388667	",
                "	18861312783	",
                "	15169691322	",
                "	15890896131	",
                "	15936251225	",
                "	18351992591	",
                "	17342603954	",
                "	18149752041	",
                "	18443187399	",
                "	15358526608	",
                "	18251397828	",
                "	15201052538	",
                "	18241078683	",
                "	15800603308	",
                "	13906281603	",
                "	13914690214	",
                "	18930552027	",
                "	18670366453	",
                "	19834536420	",
                "	18916278679	",
                "	18850013725	",
                "	18281988296	",
                "	17715010512	",
                "	13034330306	",
                "	15639178630	",
                "	15151410040	",
                "	18683557001	",
                "	18800325112	",
                "	18223239856	",
                "	15821505138	",
                "	18521318582	",
                "	15136106299	",
                "	15800517871	",
                "	15855711723	",
                "	13526546275	",
                "	13327966908	",
                "	18717869629	",
                "	13051191212	",
                "	17573202652	",
                "	15201861971	",
                "	15895966758	",
                "	13476553636	",
                "	15167892370	",
                "	13182871260	",
                "	15139400563	",
                "	15882701762	",
                "	15921870696	",
                "	13602265359	",
                "	18706012782	",
                "	13934960719	",
                "	14704547100	",
                "	17621172137	",
                "	18003052543	",
                "	15618930965	",
                "	19121740253	",
                "	19820220469	",
                "	15002188757	",
                "	13962983803	",
                "	18451558979	",
                "	13641758221	",
                "	18469106108	",
                "	18116196875	",
                "	18823006025	",
                "	15862593131	",
                "	15221099718	",
                "	18726245576	",
                "	18621328560	",
                "	17839175020	",
                "	16601970901	",
                "	13856887441	",
                "	18955510425	",
                "	18711055300	",
                "	17392926525	",
                "	15861882525	",
                "	18861120613	",
                "	15205116641	",
                "	15725526063	",
                "	16674200718	",
                "	15665166935	",
                "	18032424502	",
                "	13419133241	",
                "	18163979302	",
                "	18816294368	",
                "	13588506635	",
                "	13451569600	",
                "	17738709104	",
                "	15929753264	",
                "	13940964730	",
                "	13382865298	",
                "	18379196911	",
                "	19121806261	",
                "	13661944719	",
                "	15821356323	",
                "	18800207820	",
                "	13375183936	",
                "	15250967918	",
                "	13761019930	",
                "	18217208636	",
                "	13914084011	",
                "	15161033000	",
                "	13231623534	",
                "	15776803581	",
                "	13332245005	",
                "	15399656709	",
                "	15057158233	",
                "	18501795407	",
                "	15152480850	",
                "	16638229445	",
                "	15262466250	",
                "	18559129807	",
                "	18851304600	",
                "	18988229586	",
                "	18810586169	",
                "	13817036405	",
                "	13246283419	",
                "	15532597228	",
                "	15692103110	",
                "	19921941005	",
                "	13341983495	",
                "	13262295319	",
                "	15205600900	",
                "	13661822313	",
                "	15978827730	",
                "	13770590020	",
                "	18156304322	",
                "	15155069399	",
                "	18915092291	",
                "	17681992206	",
                "	15117621717	",
                "	13915653593	",
                "	13827427119	",
                "	13501744395	",
                "	15101609814	",
                "	13381998107	",
                "	18101525513	",
                "	15665327389	",
                "	15255602273	",
                "	13918996431	",
                "	13298399593	",
                "	17681745210	",
                "	17721640792	",
                "	13525319507	",
                "	18896737595	",
                "	13817762644	",
                "	18795973586	",
                "	15088662215	",
                "	18362668387	",
                "	13386504566	",
                "	15800505056	",
                "	15921870791	",
                "	15803641142	",
                "	18217183563	",
                "	17826110985	",
                "	15295691771	",
                "	18623045066	",
                "	15862517892	",
                "	18760959031	",
                "	17717829910	",
                "	18028385615	",
                "	15621607337	",
                "	18961412517	",
                "	13564380056	",
                "	18701739719	",
                "	18661216718	",
                "	13661949594	",
                "	17863450430	",
                "	13815296335	",
                "	18158878115	",
                "	15850376510	",
                "	17630806361	",
                "	15050611383	",
                "	15202123898	",
                "	15921931710	",
                "	18795890973	",
                "	13524532263	",
                "	18817227972	",
                "	18013515769	",
                "	15239583934	",
                "	13086030666	",
                "	18855059761	",
                "	15021058829	",
                "	15902177390	",
                "	18322596615	",
                "	18768847317	",
                "	18842319557	",
                "	13329739802	",
                "	18756050809	",
                "	13023215910	",
                "	18930733642	",
                "	18516071497	",
                "	18841860620	",
                "	18015616767	",
                "	13670249363	",
                "	18302121192	",
                "	17756936772	",
                "	19121691617	",
                "	18752824090	",
                "	13818345396	",
                "	13484637328	",
                "	18896709865	",
                "	15852923919	",
                "	13861034332	",
                "	15256299404	",
                "	15605064412	",
                "	18262292904	",
                "	15900995419	",
                "	18317080735	",
                "	13151265039	",
                "	19850093316	",
                "	18150160956	",
                "	15210802250	",
                "	13547781957	",
                "	15901862331	",
                "	17312136637	",
                "	18999100966	",
                "	13818874599	",
                "	13524239749	",
                "	15221015480	",
                "	18717872819	",
                "	18603602237	",
                "	15251365200	",
                "	13262592296	",
                "	13611784669	",
                "	18895336216	",
                "	13952232110	",
                "	18209921298	",
                "	18206123710	",
                "	15618050418	",
                "	18930654591	",
                "	15301828835	",
                "	13843967276	",
                "	13917246892	",
                "	13001375962	",
                "	13020195052	",
                "	13697916609	",
                "	17315146181	",
                "	15039959355	",
                "	18339043733	",
                "	18621578984	",
                "	18816925329	",
                "	17738408209	",
                "	18370204600	",
                "	13761406599	",
                "	18112530257	",
                "	13008927193	",
                "	18516144621	",
                "	18328668023	",
                "	13021169033	",
                "	15861175105	",
                "	13122378796	",
                "	18321173263	",
                "	15861195300	",
                "	15909825972	",
                "	15890845029	",
                "	16621165215	",
                "	18321157082	",
                "	13548360245	",
                "	18516133261	",
                "	18139990801	",
                "	18764484252	",
                "	13962364009	",
                "	18916139179	",
                "	13262851712	",
                "	13524566961	",
                "	13568959015	",
                "	18711003519	",
                "	13167212623	",
                "	13601931060	",
                "	15104068083	",
                "	15051145236	",
                "	18302185702	",
                "	13588231453	",
                "	13262538016	",
                "	15111273607	",
                "	18717164434	",
                "	18217622796	",
                "	18855440238	",
                "	18395563550	",
                "	18621950972	",
                "	17865567185	",
                "	13997713211	",
                "	13248127090	",
                "	15829000126	",
                "	13012575998	",
                "	15962703887	",
                "	13585260147	",
                "	18501517756	",
                "	13916082622	",
                "	18943579961	",
                "	15935437097	",
                "	18800200370	",
                "	16628705509	",
                "	18296094627	",
                "	18260282161	",
                "	17858323523	",
                "	18817252490	",
                "	18621976318	",
                "	18338191903	",
                "	17696858713	",
                "	18156098510	",
                "	18055718244	",
                "	15179499760	",
                "	15062595512	",
                "	13661795079	",
                "	18556022221	",
                "	15617989823	",
                "	15921612328	",
                "	18806134974	",
                "	13611910975	",
                "	13013886219	",
                "	17763368607	",
                "	15928087953	",
                "	18321208920	",
                "	18257377524	",
                "	14755541852	",
                "	17352462375	",
                "	17317734850	",
                "	15800960143	",
                "	13285658760	",
                "	17851552135	",
                "	13122880302	",
                "	13810476797	",
                "	18834845535	",
                "	13761465917	",
                "	13196225859	",
                "	15356181070	",
                "	13861159078	",
                "	17704105577	",
                "	18202116998	",
                "	18936241599	",
                "	13027134712	",
                "	18217225285	",
                "	18337982329	",
                "	18154039382	",
                "	15073876576	",
                "	18316535520	",
                "	18921016606	",
                "	13301498382	",
                "	17671675135	",
                "	18701991808	",
                "	17355437499	",
                "	18049986018	",
                "	13768531454	",
                "	18153323387	",
                "	13262217695	",
                "	13237988835	",
                "	19981490025	",
                "	19952755381	",
                "	17301703008	",
                "	15295305680	",
                "	15189802952	",
                "	13375267219	",
                "	18262892369	",
                "	13581872273	",
                "	18768187343	",
                "	18801769686	",
                "	15618583212	",
                "	13601852020	",
                "	15901812446	",
                "	13681459126	",
                "	13795490993	",
                "	18621125885	",
                "	18795852018	",
                "	13661905864	",
                "	18817751608	",
                "	18652724323	",
                "	13917209332	",
                "	17872493175	",
                "	15216650688	",
                "	13564519970	",
                "	13116627370	",
                "	15601898957	",
                "	13764545720	",
                "	18531135126	",
                "	17612188854	",
                "	13917479063	",
                "	13916566940	",
                "	15943030558	",
                "	13661774101	",
                "	17375068667	",
                "	18800252920	",
                "	13532076600	",
                "	13062631228	",
                "	18858345606	",
                "	18616585868	",
                "	18057162867	",
                "	17857181535	",
                "	18809482152	",
                "	18351262689	",
                "	18702145083	",
                "	18706197063	",
                "	18297966821	",
                "	17717411986	",
                "	18727089026	",
                "	19821838910	",
                "	19957848507	",
                "	13918285668	",
                "	18142348236	",
                "	18701842774	",
                "	19821228947	",
                "	15278258355	",
                "	18118135771	",
                "	13122258879	",
                "	15694438351	",
                "	17621708399	",
                "	19850079392	",
                "	15004024873	",
                "	15106285202	",
                "	13311685276	",
                "	17854255706	",
                "	15079918128	",
                "	15009136303	",
                "	13770461857	",
                "	19121751320	",
                "	15155187915	",
                "	15604430258	",
                "	17616242154	",
                "	13705056493	",
                "	15216373327	",
                "	18721032587	",
                "	15872872425	",
                "	18991418726	",
                "	15706216841	",
                "	15951653662	",
                "	15770938970	",
                "	13262267601	",
                "	13817936612	",
                "	15301998210	",
                "	15122532949	",
                "	13972745505	",
                "	18715604984	",
                "	18895313416	",
                "	13958380326	",
                "	17302114442	",
                "	15710693521	",
                "	13032213059	",
                "	15715791050	",
                "	18742665994	",
                "	15061035775	",
                "	18260097506	",
                "	13761062706	",
                "	15565013639	",
                "	15919625468	",
                "	19946037120	",
                "	18792497376	",
                "	13957315772	",
                "	13159566266	",
                "	15827529381	",
                "	17720808316	",
                "	17314993843	",
                "	15903024047	",
                "	13402079956	",
                "	15711015221	",
                "	15851616810	",
                "	18930516475	",
                "	15150572683	",
                "	18638577680	",
                "	18016320185	",
                "	18851193660	",
                "	18845635954	",
                "	15800961676	",
                "	17851307856	",
                "	18721971547	",
                "	13636320437	",
                "	18730383502	",
                "	13208849873	",
                "	18895700686	",
                "	18043867655	",
                "	15921599130	",
                "	18018598761	",
                "	13335115020	",
                "	18569375499	",
                "	13819022430	",
                "	15162175176	",
                "	13918034871	",
                "	17865182899	",
                "	15162868023	",
                "	13815054510	",
                "	13764531720	",
                "	15524377475	",
                "	18019057790	",
                "	13404108373	",
                "	18183771336	",
                "	18351075815	",
                "	13262558676	",
                "	13861271899	",
                "	18321069765	",
                "	18712375077	",
                "	18842470719	",
                "	15216666172	",
                "	15362689932	",
                "	18206219927	",
                "	18124223427	",
                "	17824242156	",
                "	18616054437	",
                "	18217668557	",
                "	18855034309	",
                "	15000712507	",
                "	18317107472	",
                "	13145770055	",
                "	17300621073	",
                "	18247948998	",
                "	18616336335	",
                "	18734292123	",
                "	13386986599	",
                "	18862216852	",
                "	15290840825	",
                "	15800910573	",
                "	18501662873	",
                "	13918235573	",
                "	18360131617	",
                "	13818214743	",
                "	15161190627	",
                "	15801865008	",
                "	13674318531	",
                "	15389326658	",
                "	13675562862	",
                "	18252993458	",
                "	13122832078	",
                "	18856962119	",
                "	15002136870	",
                "	18687550877	",
                "	15556992809	",
                "	18857210718	",
                "	13764858696	",
                "	19822796274	",
                "	18217631932	",
                "	17317237783	",
                "	13564162802	",
                "	13647314293	",
                "	17766082692	",
                "	18800248774	",
                "	18019115520	",
                "	18302996610	",
                "	13621968575	",
                "	18217740026	",
                "	18202133307	",
                "	18621020517	",
                "	18874314519	",
                "	15274978120	",
                "	18321771522	",
                "	15821753612	",
                "	15974820211	",
                "	17835714402	",
                "	15252578667	",
                "	18135900130	",
                "	15724417625	",
                "	13122076830	",
                "	17803869908	",
                "	13671616768	",
                "	18658856347	",
                "	17710703850	",
                "	15669331913	",
                "	18260635926	",
                "	18912221859	",
                "	15545499792	",
                "	15121069053	",
                "	18367625260	",
                "	18870830421	",
                "	13033565898	",
                "	18838939520	",
                "	18817200658	",
                "	15287084055	",
                "	15850679926	",
                "	18800205487	",
                "	15996239377	",
                "	15715930827	",
                "	13262788960	",
                "	15722934520	",
                "	18795899269	",
                "	13262789916	",
                "	18373749868	",
                "	17730211109	",
                "	15000649306	",
                "	13420137700	",
                "	18434375448	",
                "	13056082036	",
                "	18735341996	",
                "	13817491518	",
                "	15251407199	",
                "	18295188186	",
                "	15026622129	",
                "	18501706229	",
                "	15221805094	",
                "	15095361065	",
                "	18301755839	",
                "	17693523612	",
                "	17826613692	",
                "	18075962735	",
                "	18726839309	",
                "	18862212625	",
                "	18850410173	",
                "	15978569221	",
                "	13564216318	",
                "	17805014801	",
                "	13946386586	",
                "	18516329611	",
                "	15021384526	",
                "	13915100973	",
                "	15734008860	",
                "	17315840072	",
                "	15011073947	",
                "	17308109637	",
                "	17715019970	",
                "	18851772838	",
                "	15623375891	",
                "	16621383313	",
                "	13586156477	",
                "	15195961570	",
                "	15989844519	",
                "	15962833226	",
                "	15821443874	",
                "	18662009491	",
                "	18842546973	",
                "	18888109187	",
                "	18621113831	",
                "	18752233080	",
                "	15898276117	",
                "	18210172216	",
                "	18302154571	",
                "	18717138574	",
                "	13471166246	",
                "	15821669952	",
                "	18736795216	",
                "	15710612607	",
                "	15021143691	",
                "	18004637114	",
                "	19921972801	",
                "	18883252377	",
                "	15000475636	",
                "	19512142516	",
                "	19805183007	",
                "	18071459533	",
                "	18357339297	",
                "	18560788708	",
                "	15057886389	",
                "	18262633308	",
                "	18674660162	",
                "	17317132424	",
                "	18852893665	",
                "	18756987715	",
                "	18857263988	",
                "	18651180236	",
                "	13053135320	",
                "	18306203558	",
                "	13962819526	",
                "	13761424606	",
                "	13944628750	",
                "	19821221195	",
                "	18862829815	",
                "	18069558802	",
                "	15542513323	",
                "	15868122850	",
                "	17737052210	",
                "	18340828253	",
                "	13636690742	",
                "	13186023219	",
                "	15851981605	",
                "	13823440135	",
                "	15201975075	",
                "	13661467397	",
                "	13035025586	",
                "	13726296496	",
                "	17321127023	",
                "	15618591761	",
                "	13817452060	",
                "	15626656426	",
                "	15821482688	",
                "	18151969732	",
                "	19821929439	",
                "	17623127122	",
                "	17521677024	",
                "	15720607733	",
                "	13045629737	",
                "	18964221102	",
                "	13771535857	",
                "	18621593624	",
                "	18717755216	",
                "	15821932917	",
                "	17626040557	",
                "	16621224568	",
                "	15280130807	",
                "	18516284153	",
                "	18100176706	",
                "	18247758321	",
                "	18217712171	",
                "	13052215471	",
                "	18019228303	",
                "	18937996267	",
                "	15209859846	",
                "	13816624370	",
                "	13167440503	",
                "	15395281998	",
                "	15527887885	",
                "	13057609499	",
                "	18516237728	",
                "	15937641290	",
                "	17637891587	",
                "	13262572710	",
                "	18182581083	",

            };

            var phones = tempphones.Select(x => x.Trim())
                                   //.Where(x=>!igoreList.Exists(i=>i.Trim()==x))
                                   .GroupBy(x => x).Select(x => x.FirstOrDefault())
                                   .Where(x => !string.IsNullOrEmpty(x))
                                   .Where(x => x.Length == 11)
                                   .ToList();


            foreach (var phone in phones)
            {
                try
                {
                    var aa = _zhuTongSmsService.SendSMS(phone, content, true);
                }
                catch (System.Exception e)
                {
                    Console.WriteLine(e);
                    throw;
                }

            }

            //var pats = _pubPatRepository.GetSpaiclPats();
            //foreach (var item in pats)
            //{
            //    if (!string.IsNullOrEmpty(item.Pat_Name))
            //    {
            //        try
            //        {
            //            string xing = string.Empty;
            //            string ming = string.Empty;
            //            if (item.Pat_Name.IsChinese())
            //            {
            //                xing = item.Pat_Name.Substring(0, 1);
            //                ming = item.Pat_Name.Substring(1);
            //            }
            //            else
            //            {
            //                xing = item.Pat_Name;
            //                ming = item.Pat_Name;
            //            }
            //            _pubPatRepository.UpdatePatName(item.Id, xing, ming);
            //        }
            //        catch (System.Exception)
            //        {
            //            continue;
            //        }
            //    }

            //}

            //var pats = _pubPatRepository.GetPatInfo();
            //var dis = new List<int>();
            //foreach (var item in pat)
            //{
            //    var tcpat = _tCService.GetPatient(item.Pat_MRN, "", "");
            //    if (tcpat != null && tcpat.Count > 0)
            //    {
            //        if (tcpat[0].PAPERSex.ToLower().Contains("female"))
            //        {
            //            dis.Add(item.Id);
            //        }
            //    }
            //}
            //_pubPatRepository.UpdatePatGender(dis);

            //var dasdd = Convert.ToDateTime("2020-08-08").GetAgeByBirth();
            //var sdasdad = Convert.ToDateTime("2020-10-08").GetAgeByBirth();
            //var asda = "[hos] 停车信息: B3 H区8055-8060曜影医疗专用停车位，到达入口处请出示预约短信给保安即可。如有疑问请致电4001862116。下载曜影医疗APP实时查看预约信息。";
            //asda = asda.Substring(0, asda.IndexOf("如有疑问"));

            //var pats = _pubPatRepository.GetAllPatForMrn();
            //if (pats != null)
            //{
            //    foreach (var pat in pats)
            //    {
            //        try
            //        {
            //            if (string.IsNullOrEmpty(pat.PAT_MRN))
            //            {
            //                var tcPats = _tCService.GetPatient("", pat.PHONE_NO, pat.BIRTHDAY);
            //                if (tcPats != null && tcPats.Count > 0)
            //                {
            //                    var tcPat = tcPats.Find(p => p.PAPERName == pat.LAST_NAME && p.PAPERName2 == pat.FIRST_NAME);
            //                    if (tcPat != null)
            //                    {
            //                        var res = _pubPatRepository.SavePatTcInfo(pat.ID, tcPat.PAPERRowId, tcPat.PAPMINo, tcPat.MRN);
            //                    }
            //                }
            //            }
            //            else
            //            {
            //                var tcPats = _tCService.GetPatient(pat.PAT_MRN, "", "");
            //                if (tcPats != null && tcPats.Count > 0 && !string.IsNullOrEmpty(tcPats[0].MRN))
            //                {
            //                    var res = _pubPatRepository.SavePatMrn(pat.ID, tcPats[0].MRN);
            //                }
            //            }
            //        }
            //        catch (System.Exception)
            //        {
            //            continue;
            //        }
            //    }

            //}
            //var patList = _pubPatRepository.GetMarkingPat();
            //foreach (var item in patList)
            //{
            //    _tCService.UpdatePatientPortalLink("", item.Pat_MRN, true);
            //}
        }

    }
}
